@@ -1,35 +1,49 @@
 test_that("equal to online algorithm", {
-
-  skip("long-running test")
-
+  
+  # skip("long-running test")
+  
+  # test data
+  test_roll_x <- c(lapply(test_ls, function(x){x[ , 1:3]}),
+                   list("random vector with 0's and NA's" = test_ls[[3]][ , 1]))
+  test_roll_y <- c(lapply(test_ls, function(x){x[ , 4:5]}),
+                   list("random vector with 0's and NA's" = test_ls[[3]][ , 4]))
+  test_roll_null <- c(test_roll_x, "null object" = list(NULL))
+  
+  if (requireNamespace("zoo", quietly = TRUE)) {
+    names(test_roll_x[[4]]) <- zoo::index(test_roll_x[[1]])
+    names(test_roll_y[[4]]) <- zoo::index(test_roll_y[[1]])
+  }
+  
   for (ax in 1:length(test_roll_x)) {
     for (b in 1:length(test_width)) {
-
+      
       width <- test_width[b]
       test_weights <- list(lambda ^ ((2 * width):1))
       # test_weights <- list(rep(1, width), lambda ^ (width:1), 1:width,
       #                      rep(1, 2 * width), lambda ^ ((2 * width):1), 1:(width * 2))
-
+      # test_weights <- lapply(test_weights, "-")
+      # test_weights <- list(rep(0, width))
+      
       for (c in 1:length(test_min_obs)) {
         for (d in 1:length(test_complete_obs)) {
           for (e in 1:length(test_na_restore)) {
-
+            
             expect_equal(roll_any(test_roll_x[[ax]] < 0, width,
                                   test_min_obs[c], test_complete_obs[d],
                                   test_na_restore[e], test_online[1]),
                          roll_any(test_roll_x[[ax]] < 0, width,
                                   test_min_obs[c], test_complete_obs[d],
                                   test_na_restore[e], test_online[2]))
-
+            
             expect_equal(roll_all(test_roll_x[[ax]] < 0, width,
                                   test_min_obs[c], test_complete_obs[d],
                                   test_na_restore[e], test_online[1]),
                          roll_all(test_roll_x[[ax]] < 0, width,
                                   test_min_obs[c], test_complete_obs[d],
                                   test_na_restore[e], test_online[2]))
-
+            
             for (f in 1:length(test_weights)) {
-
+              
               expect_equal(roll_sum(test_roll_x[[ax]], width,
                                     test_weights[[f]], test_min_obs[c],
                                     test_complete_obs[d], test_na_restore[e],
@@ -47,7 +61,7 @@ test_that("equal to online algorithm", {
                                      test_weights[[f]], test_min_obs[c],
                                      test_complete_obs[d], test_na_restore[e],
                                      test_online[2]))
-
+              
               expect_equal(roll_mean(test_roll_x[[ax]], width,
                                      test_weights[[f]], test_min_obs[c],
                                      test_complete_obs[d], test_na_restore[e],
@@ -56,7 +70,7 @@ test_that("equal to online algorithm", {
                                      test_weights[[f]], test_min_obs[c],
                                      test_complete_obs[d], test_na_restore[e],
                                      test_online[2]))
-
+              
               expect_equal(roll_min(test_roll_x[[ax]], width,
                                     test_weights[[f]], test_min_obs[c],
                                     test_complete_obs[d], test_na_restore[e],
@@ -102,9 +116,23 @@ test_that("equal to online algorithm", {
                                        test_weights[[f]], test_min_obs[c],
                                        test_complete_obs[d], test_na_restore[e],
                                        test_online[2]))
-
+              
+              for (g in 1:length(test_p)) {
+                
+                # "'online' is not supported"
+                expect_equal(roll_quantile(test_roll_x[[ax]],  width,
+                                           test_weights[[f]], test_p[[g]],
+                                           test_min_obs[c], test_complete_obs[d],
+                                           test_na_restore[e], test_online[1]),
+                             roll_quantile(test_roll_x[[ax]], width,
+                                           test_weights[[f]], test_p[[g]],
+                                           test_min_obs[c], test_complete_obs[d],
+                                           test_na_restore[e], test_online[2]))
+                
+              }
+              
               for (g in 1:length(test_center)) {
-
+                
                 expect_equal(roll_var(test_roll_x[[ax]], width,
                                       test_weights[[f]], test_center[g],
                                       test_min_obs[c], test_complete_obs[d],
@@ -113,7 +141,7 @@ test_that("equal to online algorithm", {
                                       test_weights[[f]], test_center[g],
                                       test_min_obs[c], test_complete_obs[d],
                                       test_na_restore[e], test_online[2]))
-
+                
                 expect_equal(roll_sd(test_roll_x[[ax]], width,
                                      test_weights[[f]], test_center[g],
                                      test_min_obs[c], test_complete_obs[d],
@@ -122,9 +150,9 @@ test_that("equal to online algorithm", {
                                      test_weights[[f]], test_center[g],
                                      test_min_obs[c], test_complete_obs[d],
                                      test_na_restore[e], test_online[2]))
-
+                
                 for (h in 1:length(test_scale)) {
-
+                  
                   expect_equal(roll_scale(test_roll_x[[ax]], width,
                                           test_weights[[f]], test_center[g],
                                           test_scale[h], test_min_obs[c],
@@ -135,9 +163,9 @@ test_that("equal to online algorithm", {
                                           test_scale[h], test_min_obs[c],
                                           test_complete_obs[d], test_na_restore[e],
                                           test_online[2]))
-
+                  
                   for (ay in 1:length(test_roll_null)) {
-
+                    
                     expect_equal(roll_cov(test_roll_x[[ax]], test_roll_null[[ay]],
                                           width, test_weights[[f]],
                                           test_center[g], test_scale[h],
@@ -148,7 +176,7 @@ test_that("equal to online algorithm", {
                                           test_center[g], test_scale[h],
                                           test_min_obs[c], test_complete_obs[d],
                                           test_na_restore[e], test_online[2]))
-
+                    
                     expect_equal(roll_cor(test_roll_x[[ax]], test_roll_null[[ay]],
                                           width, test_weights[[f]],
                                           test_center[g], test_scale[h],
@@ -159,38 +187,38 @@ test_that("equal to online algorithm", {
                                           test_center[g], test_scale[h],
                                           test_min_obs[c], test_complete_obs[d],
                                           test_na_restore[e], test_online[2]))
-
+                    
                   }
-
+                  
                 }
-
+                
               }
-
+              
               for (ay in 1:length(test_roll_y)) {
-                for (i in 1:length(test_intercept)) {
-
-                  # "'complete_obs' is not supported"
+                for (g in 1:length(test_intercept)) {
+                  
+                  # "'complete_obs = FALSE' is not supported"
                   expect_equal(roll_lm(test_roll_x[[ax]], test_roll_y[[ay]],
                                        test_width[b], test_weights[[f]],
-                                       test_intercept[i], test_min_obs[c],
+                                       test_intercept[g], test_min_obs[c],
                                        test_complete_obs[d], test_na_restore[e],
                                        test_online[1]),
                                roll_lm(test_roll_x[[ax]], test_roll_y[[ay]],
                                        test_width[b], test_weights[[f]],
-                                       test_intercept[i], test_min_obs[c],
+                                       test_intercept[g], test_min_obs[c],
                                        test_complete_obs[d], test_na_restore[e],
                                        test_online[2]))
-
+                  
                 }
               }
-
+              
             }
-
+            
           }
         }
       }
-
+      
     }
   }
-
+  
 })
\ No newline at end of file
